@@ -67,7 +67,7 @@ EXTERNAL_DEPENDENCIES = (
     ('nc', ['-h'], 'OpenBSD netcat', '', 0),
     ('vconfig', [], 'the VLAN you are talking about', '', 0),
     ('fuser', ['-V'], r'fuser \(PSmisc\)',
-     r'fuser \(PSmisc\) (\d+\.\d+)\n', "22.0"),
+     r'fuser \(PSmisc\) (\d+\.\d+|UNKNOWN)\n', "22.0"),
     ('lsof', ['-v'], r'lsof version',
      r'revision: (\d+\.\d+)\n', "4.86"),
     ('mn', ['--version'], r'\d+\.\d+.\d+',
@@ -204,6 +204,9 @@ def check_dependencies():
                 print('cannot parse version %s for %s' % (
                     version_match, required_binary))
                 return False
+            if binary == 'fuser' and binary_version == 'UNKNOWN':
+                # Workaround for psmisc 23.3
+                return True
             if version.parse(binary_version) < version.parse(binary_minversion):
                 print('%s version %s is less than required version %s' % (
                     required_binary, binary_version, binary_minversion))
@@ -255,7 +258,7 @@ def pipeline_superset_report(decoded_pcap_logs):
                     else:
                         return
                 continue
-            elif depth > 1:
+            if depth > 1:
                 section_name = section_stack[-1]
                 if table_id is not None:
                     if 'Match' in section_stack:
@@ -432,6 +435,7 @@ class FaucetResult(unittest.runner.TextTestResult): # pytype: disable=module-att
 
     root_tmpdir = None
     test_duration_secs = {}
+    unexpected_success = False
 
     def _test_tmpdir(self, test):
         return os.path.join(
@@ -449,7 +453,11 @@ class FaucetResult(unittest.runner.TextTestResult): # pytype: disable=module-att
 
     def stopTest(self, test):
         self._set_test_duration_secs(test)
-        super(FaucetResult, self).stopTest(test)
+        super().stopTest(test)
+
+    def addUnexpectedSuccess(self, test):
+        self.unexpected_success = True
+        super().addUnexpectedSuccess(test)
 
 
 class FaucetCleanupResult(FaucetResult):
@@ -460,7 +468,7 @@ class FaucetCleanupResult(FaucetResult):
         self._set_test_duration_secs(test)
         shutil.rmtree(self._test_tmpdir(test))
         self.successes.append((test, ''))
-        super(FaucetCleanupResult, self).addSuccess(test)
+        super().addSuccess(test)
 
 
 def debug_exception_handler(etype, value, trace):
@@ -563,7 +571,7 @@ def run_test_suites(debug, report_json_filename, hw_config, root_tmpdir,
     results.extend(run_parallel_test_suites(root_tmpdir, resultclass, parallel_tests))
     results.extend(run_single_test_suites(debug, root_tmpdir, resultclass, single_tests))
     report_results(results, hw_config, report_json_filename)
-    successful_results = [result for result in results if result.wasSuccessful()]
+    successful_results = [result for result in results if result.wasSuccessful() or result.unexpected_success]
     return len(results) == len(successful_results)
 
 
@@ -741,8 +749,8 @@ def parse_args():
     parser.add_argument(
         '--generative_unit', default=False, action='store_true', help='run generative unit tests')
     parser.add_argument(
-        '--generative_integration', default=False, action='store_true',
-        help='run generative integration tests')
+        '--generative_tolerance', default=False, action='store_true',
+        help='run generative integration fault-tolerance tests')
     parser.add_argument(
         '-x', help='list of test classes to exclude')
 
