@@ -105,7 +105,7 @@ class FaucetTopoTestBase(FaucetTestBase):
 
     def host_ip_address(self, host_index, vlan_index):
         """Create a string of the host IP address"""
-        if isinstance(vlan_index, list):
+        if isinstance(vlan_index, (list, tuple)):
             vlan_index = vlan_index[0]
         return '10.%u.0.%u/%u' % (vlan_index+1, host_index+1, self.NETPREFIX)
 
@@ -177,11 +177,12 @@ class FaucetTopoTestBase(FaucetTestBase):
             'routers': routers,
             'router_options': router_options,
             'include': include,
-            'include_optional': include_optional
+            'include_optional': include_optional,
+            'ignored_switches': self.IGNORED_SWITCHES
         }
         self.CONFIG = self.topo.get_config(
             n_vlans,
-            **self.configuration_options
+            **self.configuration_options,
         )
         self.mininet_host_options = mininet_host_options
         self.n_vlans = n_vlans
@@ -196,12 +197,17 @@ class FaucetTopoTestBase(FaucetTestBase):
         super().start_net()
         # Create a dictionary of host information
         self.host_information = {}
+        ips_for_vlans = {}
         for host_id, host_name in self.topo.hosts_by_id.items():
             host = self.net.get(host_name)
             vlan = self.host_vlans[host_id]
             ip_interface = None
             if vlan is not None:
-                ip = self.host_ip_address(host_id, vlan)
+                if isinstance(vlan, list):
+                    vlan = tuple(vlan)
+                ips_for_vlans.setdefault(vlan, 0)
+                ip = self.host_ip_address(ips_for_vlans[vlan], vlan)
+                ips_for_vlans[vlan] += 1
                 if self.mininet_host_options and host_id in self.mininet_host_options:
                     mininet_ip = self.mininet_host_options[host_id].get('ip', None)
                     if mininet_ip:
@@ -361,6 +367,8 @@ class FaucetTopoTestBase(FaucetTestBase):
             links = 0
             links_up = 0
             for link, ports in self.link_port_maps.items():
+                if link[0] in self.IGNORED_SWITCHES or link[1] in self.IGNORED_SWITCHES:
+                    continue
                 for port in ports:
                     dpid = self.topo.dpids_by_id[link[0]]
                     name = self.topo.switches_by_id[link[0]]
@@ -371,7 +379,6 @@ class FaucetTopoTestBase(FaucetTestBase):
             prop_up = links_up / links
             if prop_up >= prop:
                 return
-            time.sleep(1)
         self.fail('not enough links up: %f / %f' % (links_up, links))
 
     def verify_stack_down(self):
@@ -456,10 +463,11 @@ class FaucetTopoTestBase(FaucetTestBase):
             self.verify_stack_has_no_loop()
             self.flap_all_switch_ports()
 
-    def verify_tunnel_established(self, src_host, dst_host, other_host, packets=3):
+    def verify_tunnel_established(self, src_host, dst_host, other_host, packets=3, dpid=None):
         """Verify ICMP packets tunnelled from src to dst."""
         icmp_match = {'eth_type': IPV4_ETH, 'ip_proto': 1}
-        self.wait_until_matching_flow(icmp_match, table_id=self._PORT_ACL_TABLE, ofa_match=False)
+        self.wait_until_matching_flow(
+            icmp_match, table_id=self._PORT_ACL_TABLE, ofa_match=False, dpid=dpid)
         tcpdump_text = self.tcpdump_helper(
             dst_host, 'icmp[icmptype] == 8', [
                 # need to set static ARP as only ICMP is tunnelled.
@@ -469,7 +477,7 @@ class FaucetTopoTestBase(FaucetTestBase):
             packets=1, timeout=(packets + 1),
         )
         self.wait_nonzero_packet_count_flow(
-            icmp_match, table_id=self._PORT_ACL_TABLE, ofa_match=False)
+            icmp_match, table_id=self._PORT_ACL_TABLE, ofa_match=False, dpid=dpid)
         self.assertTrue(re.search(
             '%s: ICMP echo request' % other_host.IP(), tcpdump_text
         ), 'Tunnel was not established')
