@@ -162,9 +162,10 @@ class FaucetMetrics(PromClient):
         self.dp_status = self._dpid_gauge(
             'dp_status',
             'status of datapaths')
-        self.dp_root_hop_port = self._dpid_gauge(
+        self.dp_root_hop_port = self._gauge(
             'dp_root_hop_port',
-            'port that leads to stack root DP')
+            'port that leads to stack root DP',
+            self.REQUIRED_LABELS)
         self.of_dp_desc_stats = self._gauge(
             'of_dp_desc_stats',
             'DP description (OFPDescStatsReply)',
@@ -203,6 +204,17 @@ class FaucetMetrics(PromClient):
             'lacp_port_id',
             'lacp port ID for for port',
             self.PORT_REQUIRED_LABELS)
+        self.port_stack_state_change_count = self._counter(
+            'port_stack_state_change_count',
+            'number of changes in port stack state',
+            self.PORT_REQUIRED_LABELS)
+        self.port_lacp_state_change_count = self._counter(
+            'port_lacp_state_change_count',
+            'number of changes in port lacp state',
+            self.PORT_REQUIRED_LABELS)
+        self.stack_root_change_count = self._counter(
+            'stack_root_change_count',
+            'number of changes in stack root', [])
 
     def _counter(self, var, var_help, labels):
         return Counter(var, var_help, labels, registry=self._reg) # pylint: disable=unexpected-keyword-arg
