@@ -10,7 +10,7 @@ class ValveStackManager(ValveManagerBase):
     """Implement stack manager, this handles the more higher-order stack functions.
 This includes port nominations and flood directionality."""
 
-    def __init__(self, logger, dp, stack, tunnel_acls, acl_manager, **kwargs):
+    def __init__(self, logger, dp, stack, tunnel_acls, acl_manager, output_table, **kwargs):
         """
         Initialize variables and set up peer distances
 
@@ -27,6 +27,7 @@ This includes port nominations and flood directionality."""
         # Used the manage the tunnel ACLs which requires stack knowledge
         self.tunnel_acls = tunnel_acls
         self.acl_manager = acl_manager
+        self.output_table = output_table
 
         # Ports that are the shortest distance to the root
         self.towards_root_ports = None
@@ -336,21 +337,51 @@ This includes port nominations and flood directionality."""
             dst_dp, dst_port = tunnel_dest['dst_dp'], tunnel_dest['dst_port']
             # Update the tunnel rules for each tunnel action specified
             updated_sources = []
+            updated_reverse_sources = []
             for source_id, source in acl.tunnel_sources.items():
-                src_dp = source['dp']
+                # We loop through each tunnel source in a single ACL instance and update the info
+                src_dp, src_port = source['dp'], source['port']
+                in_port = self.tunnel_outport(
+                    dst_dp, src_dp, src_port)
                 out_port = self.tunnel_outport(
                     src_dp, dst_dp, dst_port)
-                if out_port:
+                updated = False
+                if out_port is None and dst_port is None and dst_dp == self.dp.name:
+                    # Will need to update at most once, to ensure the correct rules get populated in the
+                    #   destination DP for a tunnel that outputs to just a DP
                     updated = acl.update_source_tunnel_rules(
-                        self.stack.name, source_id, _id, out_port)
-                    if updated:
-                        if self.stack.name == src_dp:
-                            source_vids[source_id].append(_id)
-                        else:
-                            updated_sources.append(source_id)
+                        self.stack.name, source_id, _id, out_port, self.output_table)
+                elif out_port:
+                    updated = acl.update_source_tunnel_rules(
+                        self.stack.name, source_id, _id, out_port, self.output_table)
+                if updated:
+                    if self.stack.name == src_dp:
+                        # We need to re-build and apply the whole ACL
+                        source_vids[source_id].append(_id)
+                    else:
+                        # We only need to re-build and apply the tunnel
+                        updated_sources.append(source_id)
+                reverse_updated = False
+                if src_port is None and in_port is None and src_dp == self.dp.name:
+                    reverse_updated = acl.update_reverse_tunnel_rules(
+                        self.stack.name, source_id, _id, in_port, self.output_table)
+                elif in_port:
+                    reverse_updated = acl.update_reverse_tunnel_rules(
+                        self.stack.name, source_id, _id, in_port, self.output_table)
+                if reverse_updated:
+                    if acl.requires_reverse_tunnel(_id):
+                        # Update the reverse tunnel rules if the tunnel is configured to have them
+                        updated_reverse_sources.append(source_id)
+            # The tunnel in the ACL does not have a source on this stack instance, so
+            #   we only need to re-build the special tunnel forwarding rule.
             for source_id in updated_sources:
                 ofmsgs.extend(self.acl_manager.build_tunnel_rules_ofmsgs(
                     source_id, _id, acl))
+            for source_id in updated_reverse_sources:
+                ofmsgs.extend(self.acl_manager.build_reverse_tunnel_rules_ofmsgs(
+                    source_id, _id, acl))
+        # If a tunnel is updated, but the source is configured as the current DP
+        #   then we will also need to re-build the rest of the ACL rules aswell.
         for source_id, vids in source_vids.items():
             for vid in vids:
                 ofmsgs.extend(self.acl_manager.build_tunnel_acl_rule_ofmsgs(
@@ -364,3 +395,9 @@ This includes port nominations and flood directionality."""
             for acl in self.tunnel_acls:
                 ofmsgs.extend(self.acl_update_tunnel(acl))
         return ofmsgs
+
+    def add_port(self, port):
+        """Need to add tunnel if port comes up with tunnel ACLs."""
+        if not port.stack and port.tunnel_acls():
+            return self.add_tunnel_acls()
+        return []
