@@ -177,6 +177,7 @@ class ValvesManager:
                     self.metrics.is_dp_stack_root.labels(**labels).set(0)
                 self.meta_dp_state.stack_root_name = new_root_name
                 self.metrics.faucet_stack_root_dpid.set(new_root_valve.dp.dp_id)
+                self.metrics.stack_root_change_count.inc(1)
                 self.reload_stack_root_config(now)
                 if prev_root_name:
                     stack_change = True
@@ -192,6 +193,11 @@ class ValvesManager:
                     stack_change = True
             labels = new_root_valve.dp.base_prom_labels()
             self.metrics.is_dp_stack_root.labels(**labels).set(1)
+            for valve in valves_by_name.values():
+                labels = valve.dp.base_prom_labels()
+                path_port = valve.stack_manager.chosen_towards_port
+                path_port_number = path_port.number if path_port else 0.0
+                self.metrics.dp_root_hop_port.labels(**labels).set(path_port_number)
         if stack_change:
             for valve in valves_by_name.values():
                 valve.stale_root = True
