@@ -492,6 +492,8 @@ vlans:
             5, int(self.get_prom('port_lacp_state', labels=labels)))
         self.assertFalse(
             valve.dp.ports[1].non_stack_forwarding())
+        self.assertEqual(
+            4, int(self.get_prom('port_lacp_state_change_count_total', labels=labels)))
 
     def test_lacp_timeout(self):
         """Test LACP comes up and then times out."""
@@ -705,6 +707,86 @@ class ValveL2LearnTestCase(ValveTestBases.ValveTestNetwork):
             0, self.get_prom('learned_l2_port', labels=learn_labels))
 
 
+class SoftPipelineTestCase(ValveTestBases.ValveTestNetwork):
+    """Test warm starting match changes with soft pipeline."""
+
+    REQUIRE_TFM = False
+
+    CONFIG = """
+acls:
+    acl1:
+        - rule:
+            nw_dst: '224.0.0.5'
+            dl_type: 0x800
+            actions:
+                allow: 0
+    acl2:
+        - rule:
+            nw_dst: '224.0.0.5'
+            dl_type: 0x800
+            ip_proto: 6
+            actions:
+                allow: 0
+dps:
+    s1:
+        hardware: Open vSwitch
+        dp_id: 0x1
+        interfaces:
+            p1:
+                number: 1
+                native_vlan: 100
+                acls_in: [acl1]
+"""
+
+    def setUp(self):
+        self.setup_valves(self.CONFIG)
+
+    def test_soft(self):
+        config = yaml.load(self.CONFIG, Loader=yaml.SafeLoader)
+        config['dps']['s1']['interfaces']['p1']['acls_in'] = ['acl2']
+        # We changed match conditions only, so this can be a warm start.
+        self.update_config(yaml.dump(config), reload_type='warm')
+
+
+class HardPipelineTestCase(ValveTestBases.ValveTestNetwork):
+    """Test cold starting match conditions with hard pipeline."""
+
+    CONFIG = """
+acls:
+    acl1:
+        - rule:
+            nw_dst: '224.0.0.5'
+            dl_type: 0x800
+            actions:
+                allow: 0
+    acl2:
+        - rule:
+            nw_dst: '224.0.0.5'
+            dl_type: 0x800
+            ip_proto: 6
+            actions:
+                allow: 0
+dps:
+    s1:
+        hardware: GenericTFM
+        dp_id: 0x1
+        interfaces:
+            p1:
+                number: 1
+                native_vlan: 100
+                acls_in: [acl1]
+"""
+
+    def setUp(self):
+        self.setup_valves(self.CONFIG)
+
+    def test_hard(self):
+        config = yaml.load(self.CONFIG, Loader=yaml.SafeLoader)
+        config['dps']['s1']['interfaces']['p1']['acls_in'] = ['acl2']
+        # Changed match conditions require restart.
+        self.update_config(yaml.dump(config), reload_type='cold')
+
+
 class ValveMirrorTestCase(ValveTestBases.ValveTestBig):
     """Test ACL and interface mirroring."""
     # TODO: check mirror packets are present/correct
